@@ -18,7 +18,7 @@ from collections import defaultdict
 from datetime import datetime
 from fnmatch import fnmatch
 from importlib.abc import Loader
-from typing import Any, DefaultDict, Dict, Iterator, List, Tuple
+from typing import Any, DefaultDict, Dict, Iterator, List, Set, Tuple
 import argparse
 import base64
 import importlib.util
@@ -47,7 +47,8 @@ class TestCase():
     def __init__(self, data: Dict[str, Any]) -> None:
         """
         Args:
-            data (Dict[str, Any]): An AWS Resource representation or Log event to test the policy or rule against respectively.
+            data (Dict[str, Any]): An AWS Resource representation or Log event to test the policy
+            or rule against respectively.
         """
         self._data = data
 
@@ -157,12 +158,13 @@ def zip_analysis(args: argparse.Namespace) -> Tuple[int, str]:
     with zipfile.ZipFile(filename, 'w', zipfile.ZIP_DEFLATED) as zip_out:
         # Always zip the helpers
         analysis = []
-        files: Dict[str, Any] = {}
+        files: Set[str] = set()
         for (file_name, f_path, spec) in list(load_analysis_specs(
                 args.path)) + list(load_analysis_specs(HELPERS_LOCATION)):
             if file_name not in files:
                 analysis.append((file_name, f_path, spec))
-                files[file_name] = None
+                files.add(file_name)
+                files.add('./' + file_name)
         analysis = filter_analysis(analysis, args.filter)
         for analysis_spec_filename, dir_name, analysis_spec in analysis:
             zip_out.write(analysis_spec_filename)
@@ -246,14 +248,14 @@ def test_analysis(args: argparse.Namespace) -> Tuple[int, list]:
         list(load_analysis_specs(args.path)) +
         list(load_analysis_specs(HELPERS_LOCATION)))
 
-    if len(analysis) == 0:
+    if len(analysis) == 0 and len(global_analysis) == 0:
         return 1, ["Nothing to test in {}".format(args.path)]
 
     # Apply the filters as needed
     global_analysis = filter_analysis(global_analysis, args.filter)
     analysis = filter_analysis(analysis, args.filter)
 
-    if len(analysis) == 0:
+    if len(analysis) == 0 and len(global_analysis) == 0:
         return 1, [
             "No analyses in {} matched filters {}".format(
                 args.path, args.filter)
@@ -307,7 +309,7 @@ def test_analysis(args: argparse.Namespace) -> Tuple[int, list]:
     return int(bool(failed_tests or invalid_specs)), invalid_specs
 
 
-def print_summary(test_path: str, num_tests: int, failed_tests: List[Any],
+def print_summary(test_path: str, num_tests: int, failed_tests: Dict[str, list],
                   invalid_specs: List[Any]) -> None:
     '''Print a summary of passed, failed, and invalid specs'''
     print('--------------------------')
@@ -360,8 +362,8 @@ def filter_analysis(analysis: List[Any], filters: Dict[str, List]) -> List[Any]:
 
 
 def classify_analysis(
-    specs: List[Tuple[str, str, Any]]
-) -> Tuple[List[Any], List[Any], List[Any]]:
+    specs: List[Tuple[str, str,
+                      Any]]) -> Tuple[List[Any], List[Any], List[Any]]:
     # First determine the type of each file
     global_analysis = []
     analysis = []
@@ -418,16 +420,34 @@ def run_tests(analysis: Dict[str, Any], analysis_funcs: Dict[str, Any],
             failed_tests[analysis.get('PolicyID') or
                          analysis['RuleID']].append(unit_test['Name'])
             continue
-        test_result = 'PASS'
+
+        # using a dictionary to map between the tests and their outcomes
+        # assume the test passes (default "PASS")
+        # until failure condition is found (set to "FAIL")
+        test_result = defaultdict(lambda: 'PASS')
+        # check expected result
         if result != unit_test['ExpectedResult']:
-            test_result = 'FAIL'
+            test_result['outcome'] = 'FAIL'
+
+        # check dedup and title function return non-None
+        # Only applies to rules which match an incoming event
+        if unit_test['ExpectedResult']:
+            for func in ['dedup', 'title']:
+                if analysis_funcs.get(func):
+                    if not analysis_funcs[func](test_case):
+                        test_result[func] = 'FAIL'
+                        test_result['outcome'] = 'FAIL'
+
+        if test_result['outcome'] == 'FAIL':
             failed_tests[analysis.get('PolicyID') or
                          analysis['RuleID']].append(unit_test['Name'])
-        print('\t[{}] {}'.format(test_result, unit_test['Name']))
-        if analysis_funcs.get('title') and unit_test['ExpectedResult']:
-            print('\t\t[Title] {}'.format(analysis_funcs['title'](test_case)))
-        if analysis_funcs.get('dedup') and unit_test['ExpectedResult']:
-            print('\t\t[Dedup] {}'.format(analysis_funcs['dedup'](test_case)))
+
+        # print results
+        print('\t[{}] {}'.format(test_result['outcome'], unit_test['Name']))
+        for func in ['dedup', 'title']:
+            if analysis_funcs.get(func) and unit_test['ExpectedResult']:
+                print('\t\t[{}] [{}] {}'.format(
+                    test_result[func], func, analysis_funcs[func](test_case)))
 
     return failed_tests
 
@@ -439,7 +459,7 @@ def setup_parser() -> argparse.ArgumentParser:
         prog='panther_analysis_tool')
     parser.add_argument('--version',
                         action='version',
-                        version='panther_analysis_tool 0.3.3')
+                        version='panther_analysis_tool 0.3.5')
     subparsers = parser.add_subparsers()
 
     test_parser = subparsers.add_parser(
