@@ -54,6 +54,7 @@ POLICY_SCHEMA = Schema(
             str,
         Optional('DisplayName'):
             str,
+        Optional('OutputIds'): [str],
         Optional('Reference'):
             str,
         Optional('Runbook'):
@@ -92,11 +93,15 @@ RULE_SCHEMA = Schema(
             int,
         Optional('DisplayName'):
             str,
+        Optional('OutputIds'): [str],
         Optional('Reference'):
             str,
         Optional('Runbook'):
             str,
+        Optional('SummaryAttributes'): [str],
         Optional('Suppressions'): [str],
+        Optional('Threshold'):
+            int,
         Optional('Tags'): [str],
         Optional('Reports'): {
             str: list
